@@ -283,7 +283,267 @@ class CrabNet(nn.Module):
             print("\nModel architecture: out_dims, d_model, N, heads")
             print(f"{self.out_dims}, {self.d_model}, " f"{self.N}, {self.heads}")
             print(f"Running on compute device: {self.compute_device}")
-            print(f"Model size: {count_parameters(self)} parameters\n")
+
+    def fit(
+        self,
+        train_df: pd.DataFrame = None,
+        val_df: pd.DataFrame = None,
+        extend_features: List[str] = None,
+        data_dir: Union[str, PathLike] = join(
+            dirname(__file__), "data", "materials_data"
+        ),
+        transfer: str = None,
+    ):
+        """Fit CrabNet to training data and update hyperparams with validation data.
+
+        Parameters
+        ----------
+        train_df, val_df : pd.DataFrame, optional
+            Training and validation data with at minimum, "formula" and "target" columns
+            and optionally, "extra features" (based on names in `extend_features`). If
+            `val_df` is None, then `test_size` determines the amount of training data to
+            be split into `val_df`. By default None
+        extend_features : List[str], optional
+            Names of columns to use as extra features from `train_df` and `val_df`, by default None
+        data_dir : str, optional
+            The directory from which to load data if loading from a file rather than
+            a DataFrame. `data_dir` is only used if both `train_df` and `val_df` are
+            None. It is assumed that the files in the data directory will be named
+            `train.csv`, `val.csv`, and `test.csv`. By default join(dirname(__file__), "data", "materials_data")
+        transfer : str, optional
+            Path to the saved weights to use for transfer learning. If None, then no
+            transfer learning is performed. By default None
+        """
+        self.d_extend = 0 if extend_features is None else len(extend_features)
+
+        if self.model is None:
+            self.model = SubCrab(
+                compute_device=self.compute_device,
+                out_dims=self.out_dims,
+                d_model=self.d_model,
+                d_extend=self.d_extend,
+                N=self.N,
+                heads=self.heads,
+                pe_resolution=self.pe_resolution,
+                ple_resolution=self.ple_resolution,
+                emb_scaler=self.emb_scaler,
+                pos_scaler=self.pos_scaler,
+                pos_scaler_log=self.pos_scaler_log,
+                dim_feedforward=self.dim_feedforward,
+                dropout=self.dropout,
+            ).to(self.compute_device)
+
+        if self.verbose:
+            print(f"Model size: {count_parameters(self.model)} parameters\n")
+
+        # self.transfer_nn = TransferNetwork(512, 512)
+
+        # Train network starting at pretrained weights
+        if transfer is not None:
+            self.load_network(f"{transfer}.pth")
+            self.model_name = f"{self.mat_prop}"
+
+        (
+            train_data,
+            val_data,
+            data_size,
+            extra_train_data,
+            extra_val_data,
+        ) = self._separate_extended_features(train_df, val_df, data_dir)
+
+        self.batch_size = self._default_batch_size(self.batch_size, data_size)
+
+        assert isinstance(self.batch_size, int)
+        self._load_trainval_data(
+            self.batch_size, train_data, val_data, extra_train_data, extra_val_data
+        )
+
+        self.epochs, self.checkin, self.stepsize = self._get_epochs_checkin_stepsize(
+            self.epochs, self.checkin
+        )
+        assert isinstance(self.epochs, int)
+        assert isinstance(self.checkin, int)
+
+        self.step_count = 0
+
+        self._select_criterion(self.criterion)
+        assert self.criterion is not None
+
+        assert isinstance(self.model, SubCrab)
+        base_optim = Lamb(
+            params=self.model.parameters(),
+            lr=self.lr,
+            betas=self.betas,
+            eps=self.eps,
+            weight_decay=self.weight_decay,
+            adam=self.adam,
+            min_trust=self.min_trust,
+        )
+        optimizer = Lookahead(base_optimizer=base_optim, alpha=self.alpha, k=self.k)
+        self.optimizer = SWA(optimizer)
+
+        lr_scheduler = CyclicLR(
+            self.optimizer,
+            base_lr=self.base_lr,
+            max_lr=self.max_lr,
+            cycle_momentum=False,
+            step_size_up=self.stepsize,
+        )
+        self.lr_scheduler = lr_scheduler
+
+        self.loss_curve: dict = {"train": [], "val": []}
+
+        self.stepping = True
+        self.swa_start = 2  # start at (n/2) cycle (lr minimum)
+        self.xswa: List[int] = []
+        self.yswa: List[float] = []
+
+        self.lr_list: List[float] = []
+        self.discard_n = 3
+
+        assert isinstance(self.epochs, int)
+        assert isinstance(self.checkin, int)
+        for epoch in range(self.epochs):
+            self.epoch = epoch
+            self.epochs = self.epochs
+            self._train()
+            self.lr_list.append(self.optimizer.param_groups[0]["lr"])
+
+            if (
+                (epoch + 1) % self.checkin == 0
+                or epoch == self.epochs - 1
+                or epoch == 0
+            ):
+                self._losscurve_stats(self.epochs, epoch)
+
+                if self.losscurve:
+                    self._plot_losscurve(self.checkin)
+
+            self._track_stats(self.epochs, self.checkin, self.learningcurve, epoch)
+
+            if self.optimizer.discard_count >= self.discard_n:
+                if self.verbose:
+                    print(
+                        f"Discarded: {self.optimizer.discard_count}/{self.discard_n}weight updates, early-stopping now"
+                    )
+                self.optimizer.swap_swa_sgd()
+                break
+
+        if not (self.optimizer.discard_count >= self.discard_n):
+            self.optimizer.swap_swa_sgd()
+
+        if self.save:
+            self.save_network()
+
+    def predict(
+        self,
+        test_df: pd.DataFrame = None,
+        loader=None,
+        return_uncertainty=False,
+        return_true=False,
+    ):
+        """Predict on new data using a fitted CrabNet model.
+
+        Parameters
+        ----------
+        test_df : pd.DataFrame, optional
+            _description_, by default None
+        loader : torch.Dataloader, optional
+            The Dataloader corresponding to the test data, by default None
+        return_uncertainty : bool, optional
+            Whether to return standard deviation uncertainties. If `return_true`, then
+            `return_uncertainty` takes precendence and is returned as the second output. By default False
+        return_true : bool, optional
+            Whether to return the true values (used for comparison with the predicted
+            values). If `return_uncertainty` is also specified, then the uncertainties
+            appear before the true values (i.e. pred, std, true), by default False
+
+        Returns
+        -------
+        pred : np.array
+            Predicted values. Always returned.
+        uncert : np.array
+            Standard deviation uncertainty. Returned if `return_uncertainty`. Precedes
+            `act` if `act` is also returned.
+        act : np.array
+            True values. Returned if `return_true`. `uncert` precedes `act` if both
+            `uncert` and `act` are returned.
+
+        Raises
+        ------
+        SyntaxError
+            "Specify either data *or* loader, not neither."
+        SyntaxError
+            "Specify either data *or* loader, not both."
+        """
+        if test_df is None and loader is None:
+            raise SyntaxError("Specify either data *or* loader, not neither.")
+        elif test_df is not None and loader is None:
+            if self.extend_features is not None:
+                extra_features = test_df[self.extend_features]
+            else:
+                extra_features = None
+            self.load_data(test_df, extra_features=extra_features)
+            loader = self.data_loader
+        elif test_df is not None and loader is not None:
+            raise SyntaxError("Specify either data *or* loader, not both.")
+
+        len_dataset = len(loader.dataset)
+        n_atoms = int(len(loader.dataset[0][0]) / 2)
+        act = np.zeros(len_dataset)
+        pred = np.zeros(len_dataset)
+        uncert = np.zeros(len_dataset)
+        formulae = np.empty(len_dataset, dtype=list)
+        atoms = np.empty((len_dataset, n_atoms))
+        fractions = np.empty((len_dataset, n_atoms))
+
+        assert isinstance(self.model, SubCrab)
+        self.model.eval()
+
+        with torch.no_grad():
+            for i, batch_df in enumerate(loader):
+                # extract data
+                X, y, formula, extra_features = batch_df
+                src, frac = X.squeeze(-1).chunk(2, dim=1)
+
+                # send to device
+                src = src.to(self.compute_device, dtype=torch.long, non_blocking=True)
+                frac = frac.to(
+                    self.compute_device, dtype=self.data_type_torch, non_blocking=True
+                )
+                y = y.to(
+                    self.compute_device, dtype=self.data_type_torch, non_blocking=True
+                )
+                extra_features = extra_features.to(
+                    self.compute_device, dtype=self.data_type_torch, non_blocking=True
+                )
+
+                # predict
+                output = self.model.forward(src, frac, extra_features=extra_features)
+                prediction, uncertainty = output.chunk(2, dim=-1)
+                uncertainty = torch.exp(uncertainty) * self.scaler.std
+                prediction = self.scaler.unscale(prediction)
+                if self.classification:
+                    prediction = torch.sigmoid(prediction)
+
+                assert self.batch_size is not None
+                data_loc = slice(i * self.batch_size, i * self.batch_size + len(y), 1)
+
+                atoms[data_loc, :] = src.cpu().numpy()
+                fractions[data_loc, :] = frac.cpu().numpy()
+                act[data_loc] = y.view(-1).cpu().numpy()
+                pred[data_loc] = prediction.view(-1).cpu().detach().numpy()
+                uncert[data_loc] = uncertainty.view(-1).cpu().detach().numpy()
+                formulae[data_loc] = formula
+
+        if return_uncertainty and return_true:
+            return pred, uncert, act
+        elif return_uncertainty and not return_true:
+            return pred, uncert
+        elif not return_uncertainty and return_true:
+            return pred, act
+        else:
+            return pred
 
     def load_data(
         self,
@@ -427,154 +687,6 @@ class CrabNet(nn.Module):
         frac = frac / frac.sum(dim=1).unsqueeze(1).repeat(1, frac.shape[-1])
         return frac
 
-    def fit(
-        self,
-        train_df: pd.DataFrame = None,
-        val_df: pd.DataFrame = None,
-        extend_features: List[str] = None,
-        data_dir: Union[str, PathLike] = join(
-            dirname(__file__), "data", "materials_data"
-        ),
-        transfer: str = None,
-    ):
-        """Fit CrabNet to training data and update hyperparams with validation data.
-
-        Parameters
-        ----------
-        train_df, val_df : pd.DataFrame, optional
-            Training and validation data with at minimum, "formula" and "target" columns
-            and optionally, "extra features" (based on names in `extend_features`). If
-            `val_df` is None, then `test_size` determines the amount of training data to
-            be split into `val_df`. By default None
-        extend_features : List[str], optional
-            Names of columns to use as extra features from `train_df` and `val_df`, by default None
-        data_dir : str, optional
-            The directory from which to load data if loading from a file rather than
-            a DataFrame. `data_dir` is only used if both `train_df` and `val_df` are
-            None. It is assumed that the files in the data directory will be named
-            `train.csv`, `val.csv`, and `test.csv`. By default join(dirname(__file__), "data", "materials_data")
-        transfer : str, optional
-            Path to the saved weights to use for transfer learning. If None, then no
-            transfer learning is performed. By default None
-        """
-        self.d_extend = 0 if extend_features is None else len(extend_features)
-
-        if self.model is None:
-            self.model = SubCrab(
-                compute_device=self.compute_device,
-                out_dims=self.out_dims,
-                d_model=self.d_model,
-                d_extend=self.d_extend,
-                N=self.N,
-                heads=self.heads,
-                pe_resolution=self.pe_resolution,
-                ple_resolution=self.ple_resolution,
-                emb_scaler=self.emb_scaler,
-                pos_scaler=self.pos_scaler,
-                pos_scaler_log=self.pos_scaler_log,
-                dim_feedforward=self.dim_feedforward,
-                dropout=self.dropout,
-            ).to(self.compute_device)
-
-        # self.transfer_nn = TransferNetwork(512, 512)
-
-        # Train network starting at pretrained weights
-        if transfer is not None:
-            self.load_network(f"{transfer}.pth")
-            self.model_name = f"{self.mat_prop}"
-
-        (
-            train_data,
-            val_data,
-            data_size,
-            extra_train_data,
-            extra_val_data,
-        ) = self._separate_extended_features(train_df, val_df, data_dir)
-
-        self.batch_size = self._default_batch_size(self.batch_size, data_size)
-
-        assert isinstance(self.batch_size, int)
-        self._load_trainval_data(
-            self.batch_size, train_data, val_data, extra_train_data, extra_val_data
-        )
-
-        self.epochs, self.checkin, self.stepsize = self._get_epochs_checkin_stepsize(
-            self.epochs, self.checkin
-        )
-        assert isinstance(self.epochs, int)
-        assert isinstance(self.checkin, int)
-
-        self.step_count = 0
-
-        self._select_criterion(self.criterion)
-        assert self.criterion is not None
-
-        assert isinstance(self.model, SubCrab)
-        base_optim = Lamb(
-            params=self.model.parameters(),
-            lr=self.lr,
-            betas=self.betas,
-            eps=self.eps,
-            weight_decay=self.weight_decay,
-            adam=self.adam,
-            min_trust=self.min_trust,
-        )
-        optimizer = Lookahead(base_optimizer=base_optim, alpha=self.alpha, k=self.k)
-        self.optimizer = SWA(optimizer)
-
-        lr_scheduler = CyclicLR(
-            self.optimizer,
-            base_lr=self.base_lr,
-            max_lr=self.max_lr,
-            cycle_momentum=False,
-            step_size_up=self.stepsize,
-        )
-        self.lr_scheduler = lr_scheduler
-
-        self.loss_curve: dict = {"train": [], "val": []}
-
-        self.stepping = True
-        self.swa_start = 2  # start at (n/2) cycle (lr minimum)
-        self.xswa: List[int] = []
-        self.yswa: List[float] = []
-
-        self.lr_list: List[float] = []
-        self.discard_n = 3
-
-        assert isinstance(self.epochs, int)
-        assert isinstance(self.checkin, int)
-        for epoch in range(self.epochs):
-            self.epoch = epoch
-            self.epochs = self.epochs
-            self._train()
-            self.lr_list.append(self.optimizer.param_groups[0]["lr"])
-
-            if (
-                (epoch + 1) % self.checkin == 0
-                or epoch == self.epochs - 1
-                or epoch == 0
-            ):
-                self._losscurve_stats(self.epochs, epoch)
-
-                if self.losscurve:
-                    self._plot_losscurve(self.checkin)
-
-            self._track_stats(self.epochs, self.checkin, self.learningcurve, epoch)
-
-            if self.optimizer.discard_count >= self.discard_n:
-                if self.verbose:
-                    print(
-                        f"Discarded: {self.optimizer.discard_count}/{self.discard_n}weight updates, early-stopping now"
-                    )
-                self.optimizer.swap_swa_sgd()
-                break
-
-        if not (self.optimizer.discard_count >= self.discard_n):
-            self.optimizer.swap_swa_sgd()
-
-        if self.save:
-            self.save_network()
-
     def _get_epochs_checkin_stepsize(
         self, epochs: Optional[int], checkin: Optional[int]
     ):
@@ -622,10 +734,10 @@ class CrabNet(nn.Module):
         assert isinstance(epochs, int)
         mod = epochs % (self.epochs_step * 2)
         if mod != 0:
-            updated_epochs = epochs - mod
+            updated_epochs = epochs + (self.epochs_step * 2) - mod
             if self.verbose:
                 print(
-                    f"epochs not divisible by {self.epochs_step * 2}, "
+                    f"{epochs} epochs not divisible by {self.epochs_step * 2} (2*epochs_step), "
                     f"updating epochs to {updated_epochs} for learning"
                 )
             epochs = updated_epochs
@@ -952,116 +1064,6 @@ class CrabNet(nn.Module):
             data_size = train_data.shape[0]
         return train_data, val_data, data_size, extra_train_data, extra_val_data
 
-    def predict(
-        self,
-        test_df: pd.DataFrame = None,
-        loader=None,
-        return_uncertainty=False,
-        return_true=False,
-    ):
-        """Predict on new data using a fitted CrabNet model.
-
-        Parameters
-        ----------
-        test_df : pd.DataFrame, optional
-            _description_, by default None
-        loader : torch.Dataloader, optional
-            The Dataloader corresponding to the test data, by default None
-        return_uncertainty : bool, optional
-            Whether to return standard deviation uncertainties. If `return_true`, then
-            `return_uncertainty` takes precendence and is returned as the second output. By default False
-        return_true : bool, optional
-            Whether to return the true values (used for comparison with the predicted
-            values). If `return_uncertainty` is also specified, then the uncertainties
-            appear before the true values (i.e. pred, std, true), by default False
-
-        Returns
-        -------
-        pred : np.array
-            Predicted values. Always returned.
-        uncert : np.array
-            Standard deviation uncertainty. Returned if `return_uncertainty`. Precedes
-            `act` if `act` is also returned.
-        act : np.array
-            True values. Returned if `return_true`. `uncert` precedes `act` if both
-            `uncert` and `act` are returned.
-
-        Raises
-        ------
-        SyntaxError
-            "Specify either data *or* loader, not neither."
-        SyntaxError
-            "Specify either data *or* loader, not both."
-        """
-        if test_df is None and loader is None:
-            raise SyntaxError("Specify either data *or* loader, not neither.")
-        elif test_df is not None and loader is None:
-            if self.extend_features is not None:
-                extra_features = test_df[self.extend_features]
-            else:
-                extra_features = None
-            self.load_data(test_df, extra_features=extra_features)
-            loader = self.data_loader
-        elif test_df is not None and loader is not None:
-            raise SyntaxError("Specify either data *or* loader, not both.")
-
-        len_dataset = len(loader.dataset)
-        n_atoms = int(len(loader.dataset[0][0]) / 2)
-        act = np.zeros(len_dataset)
-        pred = np.zeros(len_dataset)
-        uncert = np.zeros(len_dataset)
-        formulae = np.empty(len_dataset, dtype=list)
-        atoms = np.empty((len_dataset, n_atoms))
-        fractions = np.empty((len_dataset, n_atoms))
-
-        assert isinstance(self.model, SubCrab)
-        self.model.eval()
-
-        with torch.no_grad():
-            for i, test_df in enumerate(loader):
-                # extract data
-                X, y, formula, extra_features = test_df
-                src, frac = X.squeeze(-1).chunk(2, dim=1)
-
-                # send to device
-                src = src.to(self.compute_device, dtype=torch.long, non_blocking=True)
-                frac = frac.to(
-                    self.compute_device, dtype=self.data_type_torch, non_blocking=True
-                )
-                y = y.to(
-                    self.compute_device, dtype=self.data_type_torch, non_blocking=True
-                )
-                extra_features = extra_features.to(
-                    self.compute_device, dtype=self.data_type_torch, non_blocking=True
-                )
-
-                # predict
-                output = self.model.forward(src, frac, extra_features=extra_features)
-                prediction, uncertainty = output.chunk(2, dim=-1)
-                uncertainty = torch.exp(uncertainty) * self.scaler.std
-                prediction = self.scaler.unscale(prediction)
-                if self.classification:
-                    prediction = torch.sigmoid(prediction)
-
-                assert self.batch_size is not None
-                data_loc = slice(i * self.batch_size, i * self.batch_size + len(y), 1)
-
-                atoms[data_loc, :] = src.cpu().numpy()
-                fractions[data_loc, :] = frac.cpu().numpy()
-                act[data_loc] = y.view(-1).cpu().numpy()
-                pred[data_loc] = prediction.view(-1).cpu().detach().numpy()
-                uncert[data_loc] = uncertainty.view(-1).cpu().detach().numpy()
-                formulae[data_loc] = formula
-
-        if return_uncertainty and return_true:
-            return pred, uncert, act
-        elif return_uncertainty and not return_true:
-            return pred, uncert
-        elif not return_uncertainty and return_true:
-            return pred, act
-        else:
-            return pred
-
     def save_network(self, model_name: str = None):
         """Save network weights to a ``.pth`` file.
 
